/* ***************************************************************** */
/*                                                                   */
/* Licensed Materials - Property of IBM                              */
/*                                                                   */
/* L-AENR-7DSDUB                                                     */
/*                                                                   */
/* Copyright IBM Corp. 2007, 2008  All Rights Reserved               */
/*                                                                   */
/* US Government Users Restricted Rights - Use, duplication or       */
/* disclosure restricted by GSA ADP Schedule Contract with           */
/* IBM Corp.                                                         */
/*                                                                   */
/* ***************************************************************** */

package com.ibm.productivity.tools.samples.documentworkflow;

import java.io.IOException;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.ISelectionService;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;
import org.osgi.framework.Bundle;

import com.ibm.productivity.tools.ui.RichDocumentViewFactory;
import com.ibm.productivity.tools.ui.views.RichDocumentView;
import com.ibm.productivity.tools.ui.views.RichDocumentViewSelection;
import com.sun.star.container.NoSuchElementException;
import com.sun.star.container.XNameAccess;
import com.sun.star.lang.WrappedTargetException;
import com.sun.star.table.XCell;
import com.sun.star.text.XText;
import com.sun.star.text.XTextTable;
import com.sun.star.text.XTextTablesSupplier;
import com.sun.star.uno.UnoRuntime;

public class ShelfView extends ViewPart {
	
	private Text ownerTxt;
	private Text statusTxt;
	private RichDocumentView selectedView = null;
	private TableViewer viewer;
	
    final static public boolean isWindowsOS = System.getProperty("os.name")
			.startsWith("Windows");
	
    private ISelectionListener selectionListener = new ISelectionListener(){

    	public void selectionChanged(IWorkbenchPart arg0, final ISelection arg1) {

    		//It is suggested to create a new Job for UNO call which will be 
    		//invoked by Symphony C++ process
    		//the selctionChanged() is called when selection happened within
    		//symphony document
    		Job job = new Job("Update workflow info") {
    			public IStatus run(IProgressMonitor progress) {
    				if ( ! ( arg1 instanceof IAdaptable) )
    					return Status.CANCEL_STATUS;

    				IAdaptable adaptable = ( IAdaptable ) arg1;

    				RichDocumentViewSelection selection = ( RichDocumentViewSelection )
    				adaptable.getAdapter(RichDocumentViewSelection.class);
    				
    				if( selection == null )
    					return Status.CANCEL_STATUS;

    				selectedView = selection.getView();
    				Object model = selectedView.getUNOModel();

    				XTextTablesSupplier tablesSupplier = ( XTextTablesSupplier ) 
    				UnoRuntime.queryInterface( XTextTablesSupplier.class, model );

    				if( tablesSupplier == null)
    					return Status.OK_STATUS;

    				XNameAccess nameAccess = tablesSupplier.getTextTables();

    				try {
    					XTextTable table = ( XTextTable )UnoRuntime.queryInterface
    					( XTextTable.class,nameAccess.getByName("Workflow"));
    					XCell cell = table.getCellByName( "B2" );
    					XText text = ( XText ) UnoRuntime.queryInterface
    					(XText.class, cell);
    					final String owner = text.getString();
    					cell = table.getCellByName("B3");
    					text = ( XText ) UnoRuntime.queryInterface(XText.class, cell);
    					final String reviewer = text.getString();

    					//Execute the SWT UI related functions in display thread
    					Display.getDefault().asyncExec(new Runnable(){

    						public void run() {
    							updateWorkflowInfo( owner, reviewer );						
    						}

    					});

    				} catch (NoSuchElementException e) {
    					e.printStackTrace();
    				} catch (WrappedTargetException e) {
    					e.printStackTrace();
    				}
    				return Status.OK_STATUS;
    			}
    		};
    		job.schedule();

    	}

    };

	private void updateWorkflowInfo(String owner, String reviewer) {

		ownerTxt.setText(owner);
		statusTxt.setText(reviewer);
		
	}
	
	public void createPartControl(Composite parent) {
		parent.setLayout(new RowLayout());
		// Group of Document Library
		int y = 10 ;
		Group doclibGrp = new Group(parent, SWT.NULL);
		doclibGrp.setText("Document Library");
		doclibGrp.setSize(390, 72);
		doclibGrp.setLocation(10, y);

		viewer = new TableViewer(doclibGrp, SWT.MULTI 
				| SWT.H_SCROLL | SWT.V_SCROLL);
		viewer.getTable().setSize(120, 60);
		viewer.getTable().setLocation(30, y + 20);
		viewer.setContentProvider(new ViewContentProvider());
		viewer.setLabelProvider(new ViewLabelProvider());
		viewer.setInput(this );
		hookDoubleClickAction();

		// Group of user information
		y = 10 + doclibGrp.getBounds().height + 7;
		Group workflowGrp = new Group(parent, SWT.NULL);
		workflowGrp.setText("Workflow");
		workflowGrp.setSize(390, 72);
		workflowGrp.setLocation(10, y);

		Label ownerLbl = new Label(workflowGrp, SWT.SHADOW_NONE 
				| SWT.RIGHT);
		ownerLbl.setText("Owner");
		ownerLbl.setSize(40, 22);
		ownerLbl.setLocation(10, 22);

		Label statusLbl = new Label(workflowGrp, SWT.SHADOW_NONE 
				| SWT.RIGHT);
		statusLbl.setText("Status");
		statusLbl.setSize(40, 22);
		statusLbl.setLocation(10, 46);

		ownerTxt = new Text(workflowGrp, SWT.BORDER | SWT.LEFT);
		ownerTxt.setBackground(new Color(this.getSite().
				getShell().getDisplay(), 255, 255, 255));
		ownerTxt.setSize(80, 16);
		ownerTxt.setLocation(100, 21);

		statusTxt = new Text(workflowGrp, SWT.BORDER | SWT.LEFT);
		statusTxt.setBackground(new Color(this.getSite().
				getShell().getDisplay(), 255, 255, 255));
		statusTxt.setSize(80, 16);
		statusTxt.setLocation(100, 46);

		Button commitBtn = new Button( workflowGrp, SWT.PUSH );
		commitBtn.setText("Commit");
		commitBtn.setSize(80, 20);
		commitBtn.setLocation(50, 67);
		commitBtn.addSelectionListener(new SelectionListener(){

			public void widgetDefaultSelected(SelectionEvent arg0) {				
			}

			public void widgetSelected(SelectionEvent arg0) {

				commitWorkflowInfo();
			}

		});
		IWorkbenchWindow window = PlatformUI.getWorkbench().
		getActiveWorkbenchWindow();
		ISelectionService service = window.getSelectionService();
		service.addSelectionListener(selectionListener);
	}
	
	private void commitWorkflowInfo( ) {
		String owner = ownerTxt.getText();
		String reviewer = statusTxt.getText();
		
		writeWorkflowInfo( owner, reviewer );
	}

	private void writeWorkflowInfo(String owner, String reviewer) {
		Object model = selectedView.getUNOModel();
		
		XTextTablesSupplier tableSupplier = ( XTextTablesSupplier ) 
		  UnoRuntime.queryInterface( XTextTablesSupplier.class, model );

		XNameAccess nameAccess = tableSupplier.getTextTables();

		try {
			XTextTable table = ( XTextTable )UnoRuntime.queryInterface
			  ( XTextTable.class,nameAccess.getByName("Workflow"));
			XCell cell = table.getCellByName( "B2" );
			XText text = ( XText ) UnoRuntime.queryInterface(
					XText.class, cell);
			text.setString(owner);
			cell = table.getCellByName("B3");
			text = ( XText ) UnoRuntime.queryInterface(XText.class, 
					cell);
			text.setString(reviewer);

		} catch (NoSuchElementException e) {
			e.printStackTrace();
		} catch (WrappedTargetException e) {
			e.printStackTrace();
		}

	}
	
	class ViewContentProvider implements IStructuredContentProvider {
		public void inputChanged(Viewer v, Object oldInput, 
				Object newInput) {
		}
		public void dispose() {
		}
		public Object[] getElements(Object parent) {
			return new String[] { "Document 1", "Document 2", 
					"Document 3" };
		}
	}
	class ViewLabelProvider extends LabelProvider implements 
		ITableLabelProvider {
		public String getColumnText(Object obj, int index) {
			return getText(obj);
		}
		public Image getColumnImage(Object obj, int index) {
			return getImage(obj);
		}
		public Image getImage(Object obj) {
			return null;
		}
	}

	private void hookDoubleClickAction() {
		viewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				ISelection selection = viewer.getSelection();
				Object obj = ((IStructuredSelection)selection).
					getFirstElement();

				String displayName = obj.toString();
				String url = getDocumentURL( displayName );

				RichDocumentViewFactory.openView( url, false );
				
			}
		});
	}

	private String getDocumentURL( String displayName ) {
		String url = "";
		
		String res = "docs/" + displayName + ".odt";
		
		url = getResolvedPath( res );
		
		return url;
	}

	private String getResolvedPath(String file) {
		String resolvedPath = null;
        Bundle bundle = Platform.getBundle(Activator.PLUGIN_ID );
        if (bundle != null) {
            java.net.URL bundleURL = bundle.getEntry( "/" );
            if (bundleURL != null) {
                try {
                    resolvedPath = FileLocator.resolve( 
                    		bundleURL ).getFile();
                } catch (IOException e) {
                	e.printStackTrace();
                }
            }
        }
        
        String ret = resolvedPath+file;
        if(isWindowsOS) {
        	ret = ret.substring(1);
        	ret = ret.replace('/', '\\');
        }
        return ret;
	}

	public void setFocus() {
		viewer.getControl().setFocus();
	}
		
}
