/* ***************************************************************** */
/*                                                                   */
/* Licensed Materials - Property of IBM                              */
/*                                                                   */
/* L-AENR-7DSDUB                                                     */
/*                                                                   */
/* Copyright IBM Corp. 2007, 2008  All Rights Reserved               */
/*                                                                   */
/* US Government Users Restricted Rights - Use, duplication or       */
/* disclosure restricted by GSA ADP Schedule Contract with           */
/* IBM Corp.                                                         */
/*                                                                   */
/* ***************************************************************** */

package com.ibm.productivity.tools.samples.convertor.richapp;

import java.io.IOException;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.action.Action;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.part.ViewPart;
import org.osgi.framework.Bundle;

import com.ibm.productivity.tools.samples.convertor.snippet.ExportGIF;
import com.ibm.productivity.tools.ui.exception.ProductivityToolsException;
import com.ibm.productivity.tools.ui.exception.UninitializedDocumentException;
import com.ibm.productivity.tools.ui.model.RichDocument;
import com.ibm.productivity.tools.ui.model.RichDocumentProvider;
import com.ibm.productivity.tools.ui.viewer.RichDocumentViewer;

public class SampleForUNOView extends ViewPart {

	
	// The main viewer
	protected RichDocumentViewer viewer;
    
	// The visible document which is contained by the viewer
    private RichDocument visibleDocument;
    
	public SampleForUNOView() {
		super();
		// TODO Auto-generated constructor stub
	}

	public void createPartControl(Composite parent)	{
    	// Creates one or more controls within the parent
        viewer = new RichDocumentViewer( parent, true );
        
        // The external file
        String fileName = getResolvedPath( "resource/sample.odp" );
        // Loads the file and creates the document
        visibleDocument = new RichDocumentProvider().createRichDocument(fileName);

        // Sets the document in the viewer
        try {
			viewer.setDocument( visibleDocument );
		} catch ( IOException e ) {
			e.printStackTrace();
		} catch ( ProductivityToolsException e ) {
			e.printStackTrace();
		}
        
        // Creates "save" action
        Action exportJPGAction = new Action() {
            public void run() {

            	try {
					ExportGIF.exportGIF( viewer.getDocument().getUNOModel(), 2, "c:\\test.gif" );
				} catch (UninitializedDocumentException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
            	
            }
        };
        exportJPGAction.setText("Export GIF");
        exportJPGAction.setToolTipText("Export GIF");
        // contributes these five actions to bar
        IActionBars bars = getViewSite().getActionBars();
        bars.getToolBarManager().add(exportJPGAction); 

               
	}

	public void setFocus() {
		// TODO Auto-generated method stub

	}	
	
    /**
     * Resolves the external file path 
     * 
     * Parameters:
	 * 		file - the external file path
	 * Returns:
	 *		the resolved path 
     */
	private String getResolvedPath(String file){
		String resolvedPath = null;
        Bundle bundle = Platform.getBundle( "com.ibm.productivity.tools.samples.UNO" );
        if (bundle != null) {
            java.net.URL bundleURL = bundle.getEntry( "/" );
            if (bundleURL != null) {
                try {
                    resolvedPath = FileLocator.resolve( bundleURL ).getFile();
                } catch (IOException e) {
                	e.printStackTrace();
                }
            }
        }
        return resolvedPath+file;
	}
}
