/* ***************************************************************** */
/*                                                                   */
/* Licensed Materials - Property of IBM                              */
/*                                                                   */
/* L-AENR-7DSDUB                                                     */
/*                                                                   */
/* Copyright IBM Corp. 2007, 2008  All Rights Reserved               */
/*                                                                   */
/* US Government Users Restricted Rights - Use, duplication or       */
/* disclosure restricted by GSA ADP Schedule Contract with           */
/* IBM Corp.                                                         */
/*                                                                   */
/* ***************************************************************** */

package com.ibm.productivity.tools.samples.convertor.snippet;

import com.sun.star.beans.PropertyValue;
import com.sun.star.document.XExporter;
import com.sun.star.document.XFilter;
import com.sun.star.drawing.XDrawPage;
import com.sun.star.drawing.XDrawPages;
import com.sun.star.drawing.XDrawPagesSupplier;
import com.sun.star.lang.XComponent;
import com.sun.star.lang.XMultiServiceFactory;
import com.sun.star.uno.UnoRuntime;

public class ExportGIF {

	public static void exportGIF(XComponent xComponent, int nPageIndex,
			String fileName) {
		try {

			XMultiServiceFactory xServiceFactory = ServiceFactory
					.getServiceFactory();

			Object GraphicExportFilter = xServiceFactory
					.createInstance("com.sun.star.drawing.GraphicExportFilter");
			XExporter xExporter = (XExporter) UnoRuntime.queryInterface(
					XExporter.class, GraphicExportFilter);

			PropertyValue aProps[] = new PropertyValue[2];
			aProps[0] = new PropertyValue();
			aProps[0].Name = "MediaType";
			aProps[0].Value = "image/gif";

			/* some graphics e.g. the Windows Metafile does not have a Media Type, for this case
			 aProps[0].Name = "FilterName";	// it is possible to set a FilterName 
			 aProps[0].Value = "WMF";
			 */
			java.io.File destFile = new java.io.File(fileName);
			StringBuffer destUrl = new StringBuffer("file:///");
			destUrl.append(destFile.getCanonicalPath().replace('\\', '/'));

			aProps[1] = new PropertyValue();
			aProps[1].Name = "URL";
			aProps[1].Value = destUrl.toString();//args[ 1 ];

			if (nPageIndex < getDrawPageCount(xComponent) && nPageIndex > 1) {
				XDrawPage xPage = getDrawPageByIndex(xComponent, nPageIndex);
				XComponent xComp = (XComponent) UnoRuntime.queryInterface(
						XComponent.class, xPage);
				xExporter.setSourceDocument(xComp);
				XFilter xFilter = (XFilter) UnoRuntime.queryInterface(
						XFilter.class, xExporter);
				xFilter.filter(aProps);
				System.out.println("*** graphics on page \"" + nPageIndex

				+ "\" exported under the name \"" + fileName
						+ "\" in the local directory");
			} else {
				System.out.println("page index not in range");
			}
		} catch (Exception ex) {
			System.out.println(ex);
		}

	}

	static public int getDrawPageCount(XComponent xComponent) {
		XDrawPagesSupplier xDrawPagesSupplier = (XDrawPagesSupplier) UnoRuntime
				.queryInterface(XDrawPagesSupplier.class, xComponent);
		XDrawPages xDrawPages = xDrawPagesSupplier.getDrawPages();
		return xDrawPages.getCount();
	}

	/** get draw page by index
	 */
	static public XDrawPage getDrawPageByIndex(XComponent xComponent, int nIndex)
			throws com.sun.star.lang.IndexOutOfBoundsException,
			com.sun.star.lang.WrappedTargetException {
		XDrawPagesSupplier xDrawPagesSupplier = (XDrawPagesSupplier) UnoRuntime
				.queryInterface(XDrawPagesSupplier.class, xComponent);
		XDrawPages xDrawPages = xDrawPagesSupplier.getDrawPages();
		return (XDrawPage) UnoRuntime.queryInterface(XDrawPage.class,
				xDrawPages.getByIndex(nIndex));
	}

}
