/* ***************************************************************** */
/*                                                                   */
/* Licensed Materials - Property of IBM                              */
/*                                                                   */
/* L-AENR-7DSDUB                                                     */
/*                                                                   */
/* Copyright IBM Corp. 2007, 2008  All Rights Reserved               */
/*                                                                   */
/* US Government Users Restricted Rights - Use, duplication or       */
/* disclosure restricted by GSA ADP Schedule Contract with           */
/* IBM Corp.                                                         */
/*                                                                   */
/* ***************************************************************** */

package com.ibm.productivity.tools.samples.customizing.util;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.ISelectionService;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import com.ibm.productivity.tools.samples.customizing.StatusbarItem;
import com.ibm.productivity.tools.samples.customizing.view.PresentationView;
import com.ibm.productivity.tools.samples.customizing.view.SpreadsheetView;
import com.ibm.productivity.tools.samples.customizing.view.WriterView;
import com.ibm.productivity.tools.ui.views.RichDocumentView;
import com.ibm.productivity.tools.ui.views.RichDocumentViewSelection;
import com.sun.star.frame.XModel;
import com.sun.star.lang.XServiceInfo;
import com.sun.star.uno.UnoRuntime;

public class CustomizingUtil {
	private static int i;
	
	public static IWorkbenchPage getPage () {
		return PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
	}
	
	public static void newWriterView() {
		IWorkbenchPage page = getPage ();
		try {
			page.showView(WriterView.VIEW_ID,"" + i++, IWorkbenchPage.VIEW_VISIBLE);			
		} catch (PartInitException e) {
			e.printStackTrace();
		}
				
	}
	
	public static void newSpreadsheetView() {
		IWorkbenchPage page = getPage ();
		try {
			page.showView(SpreadsheetView.VIEW_ID,"" + i++, IWorkbenchPage.VIEW_VISIBLE);			
		} catch (PartInitException e) {
			e.printStackTrace();
		}
	}
	
	public static void newPresentationView() {
		IWorkbenchPage page = getPage ();
		try {
			page.showView(PresentationView.VIEW_ID,"" + i++, IWorkbenchPage.VIEW_VISIBLE);					
		} catch (PartInitException e) {
			e.printStackTrace();
		}
		
	}
	
	public static void addListener() {
		ISelectionService service = PlatformUI.getWorkbench().getActiveWorkbenchWindow().
				getSelectionService();
		
		service.addSelectionListener( new ISelectionListener() {
			
			public void selectionChanged(IWorkbenchPart arg0, ISelection arg1) {
				
				IAdaptable adaptable = (IAdaptable) arg1;
				RichDocumentViewSelection selection = (RichDocumentViewSelection)
				adaptable.getAdapter(RichDocumentViewSelection.class);        				
				RichDocumentView view = selection.getView();
				XModel model = ( XModel )UnoRuntime.queryInterface( 
						XModel.class, view.getUNOModel());
				XServiceInfo xInfo = (XServiceInfo)UnoRuntime.queryInterface(
						XServiceInfo.class, model);
				
				String type = "      ";
				if(xInfo!=null) {					
					if(xInfo.supportsService("com.sun.star.text.TextDocument")) {
						type = "Document type";
					} else if(xInfo.supportsService("com.sun.star.sheet.SpreadsheetDocument")) {
						type = "Spreadsheet type";
					} else if(xInfo.supportsService("com.sun.star.presentation.PresentationDocument")) {
						type = "Presentation type";
					}
				}
				StatusbarItem.setText(type);
			}
		});
	}	

}
