/* ***************************************************************** */
/*                                                                   */
/* Licensed Materials - Property of IBM                              */
/*                                                                   */
/* L-AENR-7DSDUB                                                     */
/*                                                                   */
/* Copyright IBM Corp. 2007, 2008  All Rights Reserved               */
/*                                                                   */
/* US Government Users Restricted Rights - Use, duplication or       */
/* disclosure restricted by GSA ADP Schedule Contract with           */
/* IBM Corp.                                                         */
/*                                                                   */
/* ***************************************************************** */

package com.ibm.productivity.tools.samples.views;

import java.io.IOException;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;

/**
 * The activator class controls the plug-in life cycle
 */
public class Activator extends AbstractUIPlugin {

    // The plug-in ID
    public static final String PLUGIN_ID = "com.ibm.productivity.tools.samples.views";

    // The shared instance
    private static Activator   plugin;

    final static public boolean isWindowsOS = System.getProperty("os.name")
    	.startsWith("Windows");
    
    /**
     * The constructor
     */
    public Activator() {
        plugin = this;
    }

    /*
     * (non-Javadoc)
     * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
     */
    public void start( BundleContext context ) throws Exception {
        super.start( context );
    }

    /*
     * (non-Javadoc)
     * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
     */
    public void stop( BundleContext context ) throws Exception {
        plugin = null;
        super.stop( context );
    }

    /**
     * Returns the shared instance
     *
     * @return the shared instance
     */
    public static Activator getDefault() {
        return plugin;
    }

    /**
     * Get the full path of the specified file
     * @param file path which is relative to project 
     * @return full path of this file
     */
    public static String getResolvedPath( String file ) {
        String resolvedPath = null;

        Bundle bundle = Platform.getBundle( PLUGIN_ID );
        if ( bundle != null ) {
            java.net.URL bundleURL = bundle.getEntry( "/" );
            if ( bundleURL != null ) {
                try {
                    resolvedPath = FileLocator.resolve( bundleURL ).getFile();
                }
                catch ( IOException e ) {
                    e.printStackTrace();
                }
            }
        }
        return resolvedPath + file;
    }
    
    /**
     * Returns an image descriptor for the image file at the given
     * plug-in relative path.
     *
     * @param path the path
     * @return the image descriptor
     */
    public static ImageDescriptor getImageDescriptor(String path) {
        return AbstractUIPlugin.imageDescriptorFromPlugin(PLUGIN_ID, path);
    }

	public static String getOSPath(String path) {
		return convertBundlePath(Activator.getResolvedPath(path));
	}
	
	private static String convertBundlePath(String filePath) {
		if (filePath.length() > 0 && filePath.startsWith("/"))
			filePath = filePath.substring(1);

		if (isWindowsOS) {
			filePath = filePath.replace('/', '\\');
		} else {
			filePath = "/" + filePath;
		}

		return filePath;
	}

}
