/* ***************************************************************** */
/*                                                                   */
/* Licensed Materials - Property of IBM                              */
/*                                                                   */
/* L-AENR-7DSDUB                                                     */
/*                                                                   */
/* Copyright IBM Corp. 2007, 2008  All Rights Reserved               */
/*                                                                   */
/* US Government Users Restricted Rights - Use, duplication or       */
/* disclosure restricted by GSA ADP Schedule Contract with           */
/* IBM Corp.                                                         */
/*                                                                   */
/* ***************************************************************** */

package com.ibm.productivity.tools.samples.views.actions;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.actions.ActionFactory;

import com.ibm.productivity.tools.samples.views.Activator;

/**
 * Access to actions provided by this sample
 * 
 */
public abstract class SampleActionFactory extends ActionFactory {

	protected SampleActionFactory(String actionId) {
		super(actionId);
	}

	public static final ActionFactory DEMO1 = new SampleActionFactory("demo1") {

		public IWorkbenchAction create(IWorkbenchWindow arg0) {
			IWorkbenchAction action = new Demo1Action();
			action.setId(getId());
			action.setText("Demo1");
			action.setImageDescriptor(Activator
					.getImageDescriptor("resources/itemContexto.png"));
			action.setDisabledImageDescriptor(Activator
					.getImageDescriptor("resources/itemContexto.png"));
			action.setHoverImageDescriptor(Activator
					.getImageDescriptor("resources/itemContexto.png"));
			action.setToolTipText("Demo1");
			return action;
		}
		
	};
	
	public static final ActionFactory DEMO2 = new SampleActionFactory("demo2") {

		public IWorkbenchAction create(IWorkbenchWindow arg0) {
			IWorkbenchAction action = new Demo2Action();
			action.setId(getId());
			action.setText("Demo2");
			action.setImageDescriptor(Activator
					.getImageDescriptor("resources/itemInserirCampo.png"));
			action.setDisabledImageDescriptor(Activator
					.getImageDescriptor("resources/itemInserirCampo.png"));
			action.setHoverImageDescriptor(Activator
					.getImageDescriptor("resources/itemInserirCampo.png"));
			action.setToolTipText("Demo2");
			return action;
		}
		
	};	
	/**
	 * a demo action
	 */
	public static final ActionFactory LOAD = new SampleActionFactory("load") {
		public IWorkbenchAction create(IWorkbenchWindow window) {
			IWorkbenchAction action = new LoadAction(window);
			action.setId(getId());
			action.setText("Open");
			action.setImageDescriptor(Activator
					.getImageDescriptor("resources/clientes.png"));
			action.setDisabledImageDescriptor(Activator
					.getImageDescriptor("resources/clientes.png"));
			action.setHoverImageDescriptor(Activator
					.getImageDescriptor("resources/clientes.png"));
			action.setToolTipText("Open");
			return action;
		}
	};

	/**
	 * demo save action
	 */
	public static final ActionFactory SAVE = new SampleActionFactory("save") {
		public IWorkbenchAction create(IWorkbenchWindow window) {
			IWorkbenchAction action = new SaveAction(window);
			action.setText("Save");
			action.setId(getId());
			action.setImageDescriptor(Activator
					.getImageDescriptor("resources/InserirCampo.png"));
			action.setDisabledImageDescriptor(Activator
					.getImageDescriptor("resources/InserirCampo.png"));
			action.setHoverImageDescriptor(Activator
					.getImageDescriptor("resources/InserirCampo.png"));
			action.setToolTipText("Save");

			return action;
		}
	};

	public static final ActionFactory CLOSE = new SampleActionFactory("close") {

		public IWorkbenchAction create(IWorkbenchWindow window) {
			IWorkbenchAction action = new CloseAction(window);
			action.setText("Close");
			action.setId(getId());
			action.setImageDescriptor(Activator
					.getImageDescriptor("resources/InserirCampo.png"));
			action.setDisabledImageDescriptor(Activator
					.getImageDescriptor("resources/InserirCampo.png"));
			action.setHoverImageDescriptor(Activator
					.getImageDescriptor("resources/InserirCampo.png"));
			action.setToolTipText("Close");

			return action;

		}
		
	};
	
	//hold these factories with a map
	static final Map factories;
	static {
		factories = new HashMap();
		factories.put("load", SampleActionFactory.LOAD);
		factories.put("save", SampleActionFactory.SAVE);
	};

	/**
	 * @param actionId
	 * @return the action factory with the specified actionId
	 */
	public static ActionFactory getFactory(String actionId) {
		ActionFactory factory = (ActionFactory)factories.get(actionId);
		return factory;
	}

}
