/* ***************************************************************** */
/*                                                                   */
/* Licensed Materials - Property of IBM                              */
/*                                                                   */
/* L-AENR-7DSDUB                                                     */
/*                                                                   */
/* Copyright IBM Corp. 2007, 2008  All Rights Reserved               */
/*                                                                   */
/* US Government Users Restricted Rights - Use, duplication or       */
/* disclosure restricted by GSA ADP Schedule Contract with           */
/* IBM Corp.                                                         */
/*                                                                   */
/* ***************************************************************** */

package com.ibm.productivity.tools.samples.writer;

import java.io.IOException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;

import org.eclipse.core.runtime.Path;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.part.ViewPart;

import com.ibm.productivity.tools.ui.RichDocumentViewFactory;
import com.ibm.productivity.tools.ui.views.RichDocumentView;
import com.sun.star.beans.UnknownPropertyException;
import com.sun.star.beans.XPropertySet;
import com.sun.star.container.XNamed;
import com.sun.star.lang.WrappedTargetException;
import com.sun.star.lang.XMultiServiceFactory;
import com.sun.star.table.XCell;
import com.sun.star.text.ControlCharacter;
import com.sun.star.text.XDependentTextField;
import com.sun.star.text.XText;
import com.sun.star.text.XTextDocument;
import com.sun.star.text.XTextSection;
import com.sun.star.text.XTextTable;
import com.sun.star.uno.Exception;
import com.sun.star.uno.UnoRuntime;

public class DemoView extends ViewPart {

	private Object model;

	private XMultiServiceFactory factory;

	private XTextDocument xTextDoc;
	
	private HashMap xTables = new HashMap();

	private Button loadButton;

	public RichDocumentView docView;

	private static DemoView instance;

	private Button addContextButton;

	private Button addProjectButton;

	private Button addActionButton;

	private ComboViewer contentsCombo;

	private HashSet contents = new HashSet();

	private Text actionText;

	private HashSet projects = new HashSet();

	private ComboViewer projectsCombo;

	protected int projectID = 0;

	public DemoView() {
		super();

		instance = this;
	}

	public void createPartControl(Composite composite) {
		composite.setLayout(new GridLayout(1, false));
		createLoadGroup(composite);
		createOperateGroup(composite);
	}

	/**
	 * Create the load group for loding the demo document.
	 * @param composite
	 */
	private void createLoadGroup(Composite composite) {
		Group load = new Group(composite, SWT.NULL);
		load.setLayout(new GridLayout());
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		load.setLayoutData(gridData);
		load.setText("1:Load");

		Label label = new Label(load, SWT.NULL);
		label.setText("Load the demo document for the next\n" + "operations:");

		loadButton = new Button(load, SWT.PUSH);
		loadButton.setText("Load Demo Document");
		loadButton.setToolTipText("Load the Getting Things Done(GTD) demo document"
						+ "for the next operations:");

		gridData = new GridData(GridData.CENTER);
		loadButton.setLayoutData(gridData);
		loadButton.addSelectionListener(new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent arg0) {

			}

			public void widgetSelected(SelectionEvent event) {
				loadDocument();
			}
		});
	}

	/**
	 * Create the operation group for operating the demo document.
	 * @param composite
	 */
	private void createOperateGroup(Composite composite) {
		Group operation = new Group(composite, SWT.NULL);
		operation.setLayout(new GridLayout(3, false));
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		operation.setLayoutData(gridData);
		operation.setText("2:Operations");
		
		Label contextDescription = new Label(operation, SWT.NULL);
		gridData = new GridData();
		gridData.horizontalSpan = 3;
		contextDescription.setLayoutData(gridData);
		contextDescription.setText("Input a Content name, create a new\n" +
				"content section with this name and\n" +
				"add it to this combo list:");
		
		Label contextLabel = new Label(operation, SWT.NONE);
		contextLabel.setText("Content:");
		contentsCombo = new ComboViewer(operation, SWT.BORDER);
		gridData = new GridData();
		gridData.widthHint = 85;
		contentsCombo.getCombo().setToolTipText("input a kind of Content name");
		contentsCombo.getCombo().setLayoutData(gridData);
		contentsCombo.setContentProvider(new ArrayContentProvider());
		contentsCombo.setLabelProvider(new LabelProvider() {

			public String getText(Object obj) {
				XTextSection context = (XTextSection) obj;
				XNamed xNamed = (XNamed) UnoRuntime.queryInterface(
						XNamed.class, context);
				return (String) xNamed.getName();
			}

		});
		contentsCombo.setInput(contents);

		addContextButton = new Button(operation, SWT.PUSH);
		addContextButton.setText("Create");
		addContextButton.setToolTipText("Create a new Content section with the " +
						"name and add it to this combo list");

		addContextButton.addSelectionListener(new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent arg0) {

			}

			public void widgetSelected(SelectionEvent event) {
				if (xTextDoc == null) {
					MessageDialog.openInformation(getSite().getShell(),
							"Demo Writer",
							"Please Load the demo document first!");
					return;
				}
				
				String contentName = contentsCombo.getCombo().getText();
				String [] list = contentsCombo.getCombo().getItems();
				if((list.length == 0 && !contentName.equals("")) || 
						(list.length > 0 && !contentName.equals("") && 
								!Arrays.asList(list).contains(contentName))){
					XTextSection context = createContent(contentName);
					contents.add(context);
					contentsCombo.setInput(contents);
					contentsCombo.setSelection(new StructuredSelection(context));
				}								
			}

		});
		
		Label projectDescription = new Label(operation, SWT.NULL);
		gridData = new GridData();
		gridData.horizontalSpan = 3;
		projectDescription.setLayoutData(gridData);
		projectDescription.setText("Input a Project name and add it to\n" +
				"this combo list:");
		
		Label projectLabel = new Label(operation, SWT.NONE);
		projectLabel.setText("Project:");
		projectsCombo = new ComboViewer(operation, SWT.BORDER);
		gridData = new GridData();
		gridData.widthHint = 85;
		projectsCombo.getCombo().setToolTipText("Input a new Project name");
		projectsCombo.getCombo().setLayoutData(gridData);
		projectsCombo.setContentProvider(new ArrayContentProvider());
		projectsCombo.setLabelProvider(new LabelProvider() {

			public String getText(Object obj) {
				XPropertySet project = (XPropertySet) obj;
				try {
					return (String) project.getPropertyValue("Content");
				} catch (UnknownPropertyException e) {
					e.printStackTrace();
				} catch (WrappedTargetException e) {
					e.printStackTrace();
				}
				return "";
			}

		});
		projectsCombo.setInput(projects);

		addProjectButton = new Button(operation, SWT.PUSH | SWT.HORIZONTAL);
		gridData = new GridData();
		gridData.widthHint = 48;
		addProjectButton.setLayoutData(gridData);
		addProjectButton.setText("Add");
		addProjectButton.setToolTipText("Add the Project name to the combo list");
		addProjectButton.addSelectionListener(new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent arg0) {

			}

			public void widgetSelected(SelectionEvent event) {
				String projectName = projectsCombo.getCombo().getText();
				String [] list = projectsCombo.getCombo().getItems();
				if((list.length == 0 && !projectName.equals("")) || 
						(list.length > 0 && !projectName.equals("")&& 
								!Arrays.asList(list).contains(projectName))){
					XPropertySet project = createProject(Integer.toString(projectID++), 
							projectName);
					projects.add(project);
					projectsCombo.setInput(projects);
					projectsCombo.setSelection(new StructuredSelection(project));
				}								
			}

		});
		
		Label actionDescription = new Label(operation, SWT.NULL);
		gridData = new GridData();
		gridData.horizontalSpan = 3;
		actionDescription.setLayoutData(gridData);
		actionDescription.setText("Input a Action name and insert it\n" +
				"with project name selected into\n" +
				"content table:");
		
		Label actionLabel = new Label(operation, SWT.NONE);
		actionLabel.setText("Action:");

		gridData = new GridData();
		gridData.widthHint = 100;
		actionText = new Text(operation, SWT.BORDER);
		actionText.setLayoutData(gridData);
		actionText.setToolTipText("Input a new action name");

		addActionButton = new Button(operation, SWT.PUSH);
		addActionButton.setText("Insert");
		addActionButton.setToolTipText("Insert the Action name and selected Project " +
						"name to the selected Content table");

		addActionButton.addSelectionListener(new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent arg0) {

			}

			public void widgetSelected(SelectionEvent event) {
				if(contentsCombo.getCombo().getText().equals("")) {
					MessageDialog.openInformation(getSite().getShell(),
							"Demo Writer",
							"The Content name can not be null.");
					return;
				}
				
				XPropertySet selection = (XPropertySet) ((IStructuredSelection) projectsCombo
						.getSelection()).getFirstElement();
				if(selection == null) {
					MessageDialog.openInformation(getSite().getShell(),
							"Demo Writer",
							"The Project name can not be null and must be added to the combo list.");
					return;
				}
				
				if(actionText.getText().equals("")) {
					MessageDialog.openInformation(getSite().getShell(),
							"Demo Writer",
							"The Action name can not be null.");
					return;
				}
				createAction(actionText.getText(), selection);
			}

		});
	}

	/**
	 * create this action with the specified project.
	 * @param name action name
	 * @param projectField
	 */
	protected void createAction(String name, XPropertySet projectField) {
		String tableKey = contentsCombo.getCombo().getText();
		XTextTable table = (XTextTable) xTables.get(tableKey);
		
		int lastRow = table.getRows().getCount();
		XCell xCell = table.getCellByName("A" + Integer.toString(lastRow));
		XText cellText = (XText) UnoRuntime.queryInterface(XText.class, xCell);
		cellText.setString(name);
		xCell = table.getCellByName("B" + Integer.toString(lastRow));
		cellText = (XText) UnoRuntime.queryInterface(XText.class, xCell);

		Object oUserField;
		try {
			oUserField = factory
					.createInstance("com.sun.star.text.TextField.User");
			XDependentTextField xUserField = (XDependentTextField) UnoRuntime
					.queryInterface(XDependentTextField.class, oUserField);
			xUserField.attachTextFieldMaster(projectField);

			cellText.insertTextContent(cellText.getStart(), xUserField, false);
			table.getRows().insertByIndex(table.getRows().getCount(), 1);

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * create a project with the name
	 * @param id project's ID
	 * @param name
	 * @return 
	 */
	private XPropertySet createProject(String id, Object name) {
		try {
			Object oUserFieldMaster = factory
					.createInstance("com.sun.star.text.FieldMaster.User");
			XPropertySet xUserFieldMaster = (XPropertySet) UnoRuntime
					.queryInterface(XPropertySet.class, oUserFieldMaster);
			// Set the name and value of the FieldMaster
			xUserFieldMaster.setPropertyValue("Name", id);
			xUserFieldMaster.setPropertyValue("Content", name);
			projects.add(xUserFieldMaster);
			return xUserFieldMaster;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * create this action into the content table
	 * @param xSection the section
	 * @param name the action's name
	 */
	protected void createActionTable(XTextSection xSection, String name) {
		try {
			Object oTable = factory
					.createInstance("com.sun.star.text.TextTable");
			XTextTable xTable = (XTextTable) UnoRuntime.queryInterface(XTextTable.class,
					oTable);
			xTables.put(name, xTable);
			xTextDoc.getText().insertTextContent(xSection.getAnchor().getEnd(),
					xTable, false);
			XText xCellText = (XText) UnoRuntime.queryInterface(XText.class,
					xTable.getCellByName("A1"));
			xCellText.setString("Name");
			xCellText = (XText) UnoRuntime.queryInterface(XText.class, xTable
					.getCellByName("B1"));
			xCellText.setString("Project");
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	/**
	 * create a content with the name
	 * @param name the content's name
	 * @return a XTextSection object
	 */
	protected XTextSection createContent(String name) {
		try {
			Object oSection = factory
					.createInstance("com.sun.star.text.TextSection");
			XTextSection xSection = (XTextSection) UnoRuntime.queryInterface(
					XTextSection.class, oSection);
			XNamed xNamed = (XNamed) UnoRuntime.queryInterface(XNamed.class,
					oSection);
			xNamed.setName(name);
			xTextDoc.getText().insertTextContent(xTextDoc.getText().getEnd(),
					xSection, false);
			xTextDoc.getText().insertString(xSection.getAnchor().getStart(),
					"Content: " + name, false);
			xTextDoc.getText().insertControlCharacter(
					xSection.getAnchor().getEnd(),
					ControlCharacter.PARAGRAPH_BREAK, false);
			xTextDoc.getText().insertString(xSection.getAnchor().getEnd(),
					"Table of actions :", false);
			createActionTable(xSection, name);
			return xSection;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * load the demo document. 
	 */
	protected void loadDocument() {
		try {
			String path = new Path(Activator.getResolvedURL(
					"/resource/sample.odt").getPath()).toOSString();
			docView = RichDocumentViewFactory.openView(path, false);			
			if(docView != null) {
				model = docView.getUNOModel();
				factory = (XMultiServiceFactory) UnoRuntime.queryInterface(
						XMultiServiceFactory.class, model);
				xTextDoc = (XTextDocument) UnoRuntime.queryInterface(
						XTextDocument.class, model);
			}			
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * do cleaning job after the document was closed.
	 */
	public void clean() {
		xTextDoc = null;
		contentsCombo.setInput(null);
		contents.clear();
		projectsCombo.setInput(null);
		projects.clear();
		actionText.setText("");
		xTables.clear();
	}

	public void setFocus() {

	}

	public static DemoView getInstance() {
		return instance;
	}

}
